"use strict";(self.rspackChunk_jupyterlab_application_top=self.rspackChunk_jupyterlab_application_top||[]).push([[753],{3112(r,t,n){function u(r,t){return r.skipToEnd(),t.cur=p,"error"}function e(r,t){return r.match(/^HTTP\/\d\.\d/)?(t.cur=c,"keyword"):r.match(/^[A-Z]+/)&&/[ \t]/.test(r.peek())?(t.cur=a,"keyword"):u(r,t)}function c(r,t){var n=r.match(/^\d+/);if(!n)return u(r,t);t.cur=o;var e=Number(n[0]);return e>=100&&e<400?"atom":"error"}function o(r,t){return r.skipToEnd(),t.cur=p,null}function a(r,t){return r.eatWhile(/\S/),t.cur=i,"string.special"}function i(r,t){return r.match(/^HTTP\/\d\.\d$/)?(t.cur=p,"keyword"):u(r,t)}function p(r){return!r.sol()||r.eat(/[ \t]/)?(r.skipToEnd(),"string"):r.match(/^.*?:/)?"atom":(r.skipToEnd(),"error")}function s(r){return r.skipToEnd(),null}n.d(t,{http:()=>l});let l={name:"http",token:function(r,t){var n=t.cur;return n!=p&&n!=s&&r.eatSpace()?null:n(r,t)},blankLine:function(r){r.cur=s},startState:function(){return{cur:e}}}}}]);
//# sourceMappingURL=753.8f5bdae92c5baced.js.map?v=8f5bdae92c5baced