"use strict";(self.rspackChunk_jupyterlab_application_top=self.rspackChunk_jupyterlab_application_top||[]).push([[1447],{21148(e,a,l){l.d(a,{K:()=>t});var r=Object.defineProperty,t=(e,a)=>r(e,"name",{value:a,configurable:!0})},96986(e,a,l){l.d(a,{default:()=>t});var r=(0,l(21148).K)(async()=>await Promise.all([l.e(2121),l.e(9518),l.e(4324),l.e(7208)]).then(l.bind(l,72847)),"loader"),t=[{name:"elk",loader:r,algorithm:"elk.layered"},...["elk.stress","elk.force","elk.mrtree","elk.sporeOverlap"].map(e=>({name:e,loader:r,algorithm:e}))]}}]);
//# sourceMappingURL=1447.9615e3d902f2299c.js.map?v=9615e3d902f2299c