(self.rspackChunk_jupyterlab_application_top=self.rspackChunk_jupyterlab_application_top||[]).push([[2080],{67901(t,e,o){var r,n=1/0,p=/[&<>"'`]/g,l=RegExp(p.source),c="object"==typeof o.g&&o.g&&o.g.Object===Object&&o.g,u="object"==typeof self&&self&&self.Object===Object&&self,i=c||u||Function("return this")(),f=(r={"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;","`":"&#96;"},function(t){return null==r?void 0:r[t]}),a=Object.prototype.toString,s=i.Symbol,b=s?s.prototype:void 0,y=b?b.toString:void 0;t.exports=function(t){var e;return(t=null==(e=t)?"":function(t){if("string"==typeof t)return t;if(function(t){return"symbol"==typeof t||!!t&&"object"==typeof t&&"[object Symbol]"==a.call(t)}(t))return y?y.call(t):"";var e=t+"";return"0"==e&&1/t==-n?"-0":e}(e))&&l.test(t)?t.replace(p,f):t}}}]);
//# sourceMappingURL=2080.afb140ad5118d6ca.js.map?v=afb140ad5118d6ca