"use strict";(self.rspackChunk_jupyterlab_application_top=self.rspackChunk_jupyterlab_application_top||[]).push([[4251],{97318(e,t,r){function n(e,t,r){return t(r),r(e,t)}r.d(t,{elm:()=>g});var i=/[a-z]/,a=/[A-Z]/,o=/[a-zA-Z0-9_]/,u=/[0-9]/,f=/[0-9A-Fa-f]/,s=/[-&*+.\\/<>=?^|:]/,l=/[(),[\]{}]/,c=/[ \v\f]/;function p(){return function(e,t){if(e.eatWhile(c))return null;var r=e.next();if(l.test(r))return"{"===r&&e.eat("-")?n(e,t,function e(t){return 0==t?p():function(r,n){for(;!r.eol();){var i=r.next();if("{"==i&&r.eat("-"))++t;else if("-"==i&&r.eat("}")&&0==--t)return n(p()),"comment"}return n(e(t)),"comment"}}(1)):"["===r&&e.match("glsl|")?n(e,t,x):"builtin";if("'"===r)return n(e,t,h);if('"'===r)return e.eat('"')?e.eat('"')?n(e,t,k):"string":n(e,t,m);if(a.test(r))return e.eatWhile(o),"type";if(i.test(r)){var d=1===e.pos;return e.eatWhile(o),d?"def":"variable"}if(u.test(r)){if("0"===r){if(e.eat(/[xX]/))return e.eatWhile(f),"number"}else e.eatWhile(u);return e.eat(".")&&e.eatWhile(u),e.eat(/[eE]/)&&(e.eat(/[-+]/),e.eatWhile(u)),"number"}return s.test(r)?"-"===r&&e.eat("-")?(e.skipToEnd(),"comment"):(e.eatWhile(s),"keyword"):"_"===r?"keyword":"error"}}function k(e,t){for(;!e.eol();)if('"'===e.next()&&e.eat('"')&&e.eat('"')){t(p());break}return"string"}function m(e,t){for(;e.skipTo('\\"');)e.next(),e.next();return e.skipTo('"')?(e.next(),t(p()),"string"):(e.skipToEnd(),t(p()),"error")}function h(e,t){for(;e.skipTo("\\'");)e.next(),e.next();return e.skipTo("'")?(e.next(),t(p()),"string"):(e.skipToEnd(),t(p()),"error")}function x(e,t){for(;!e.eol();)if("|"===e.next()&&e.eat("]")){t(p());break}return"string"}var d={case:1,of:1,as:1,if:1,then:1,else:1,let:1,in:1,type:1,alias:1,module:1,where:1,import:1,exposing:1,port:1};let g={name:"elm",startState:function(){return{f:p()}},copyState:function(e){return{f:e.f}},token:function(e,t){var r=t.f(e,function(e){t.f=e}),n=e.current();return d.hasOwnProperty(n)?"keyword":r},languageData:{commentTokens:{line:"--"}}}}}]);
//# sourceMappingURL=4251.5e623ebd8970044f.js.map?v=5e623ebd8970044f