"use strict";(self.rspackChunk_jupyterlab_application_top=self.rspackChunk_jupyterlab_application_top||[]).push([[2004],{86831(e,t,n){n.d(t,{fcl:()=>p});var r={term:!0,method:!0,accu:!0,rule:!0,then:!0,is:!0,and:!0,or:!0,if:!0,default:!0},o={var_input:!0,var_output:!0,fuzzify:!0,defuzzify:!0,function_block:!0,ruleblock:!0},a={end_ruleblock:!0,end_defuzzify:!0,end_function_block:!0,end_fuzzify:!0,end_var:!0},i={true:!0,false:!0,nan:!0,real:!0,min:!0,max:!0,cog:!0,cogs:!0},u=/[+\-*&^%:=<>!|\/]/;function l(e,t){var n=e.next();if(/[\d\.]/.test(n))return"."==n?e.match(/^[0-9]+([eE][\-+]?[0-9]+)?/):"0"==n?e.match(/^[xX][0-9a-fA-F]+/)||e.match(/^0[0-7]+/):e.match(/^[0-9]*\.?[0-9]*([eE][\-+]?[0-9]+)?/),"number";if("/"==n||"("==n){if(e.eat("*"))return t.tokenize=c,c(e,t);if(e.eat("/"))return e.skipToEnd(),"comment"}if(u.test(n))return e.eatWhile(u),"operator";e.eatWhile(/[\w\$_\xa1-\uffff]/);var l=e.current().toLowerCase();return r.propertyIsEnumerable(l)||o.propertyIsEnumerable(l)||a.propertyIsEnumerable(l)?"keyword":i.propertyIsEnumerable(l)?"atom":"variable"}function c(e,t){for(var n,r=!1;n=e.next();){if(("/"==n||")"==n)&&r){t.tokenize=l;break}r="*"==n}return"comment"}function f(e,t,n,r,o){this.indented=e,this.column=t,this.type=n,this.align=r,this.prev=o}let p={name:"fcl",startState:function(e){return{tokenize:null,context:new f(-e,0,"top",!1),indented:0,startOfLine:!0}},token:function(e,t){var n,r,i=t.context;if(e.sol()&&(null==i.align&&(i.align=!1),t.indented=e.indentation(),t.startOfLine=!0),e.eatSpace())return null;var u=(t.tokenize||l)(e,t);if("comment"==u)return u;null==i.align&&(i.align=!0);var c=e.current().toLowerCase();if(o.propertyIsEnumerable(c))r=e.column(),t.context=new f(t.indented,r,"end_block",null,t.context);else{a.propertyIsEnumerable(c)&&(n=t).context.prev&&("end_block"==n.context.type&&(n.indented=n.context.indented),n.context=n.context.prev)}return t.startOfLine=!1,u},indent:function(e,t,n){if(e.tokenize!=l&&null!=e.tokenize)return 0;var r=e.context,o=a.propertyIsEnumerable(t);return r.align?r.column+ +!o:r.indented+(o?0:n.unit)},languageData:{commentTokens:{line:"//",block:{open:"(*",close:"*)"}}}}}}]);
//# sourceMappingURL=2004.9fc4a54790880fe7.js.map?v=9fc4a54790880fe7