"use strict";(self.rspackChunk_jupyterlab_application_top=self.rspackChunk_jupyterlab_application_top||[]).push([[173],{7260(e,t,n){function r(e){return RegExp("^(("+e.join(")|(")+"))\\b")}n.d(t,{octave:()=>k});var a=RegExp("^[\\+\\-\\*/&|\\^~<>!@'\\\\]"),i=RegExp("^[\\(\\[\\{\\},:=;\\.]"),o=RegExp("^((==)|(~=)|(<=)|(>=)|(<<)|(>>)|(\\.[\\+\\-\\*/\\^\\\\]))"),c=RegExp("^((!=)|(\\+=)|(\\-=)|(\\*=)|(/=)|(&=)|(\\|=)|(\\^=))"),s=RegExp("^((>>=)|(<<=))"),l=RegExp("^[\\]\\)]"),m=RegExp("^[_A-Za-z\xa1-][_A-Za-z0-9\xa1-]*"),u=r(["error","eval","function","abs","acos","atan","asin","cos","cosh","exp","log","prod","sum","log10","max","min","sign","sin","sinh","sqrt","tan","reshape","break","zeros","default","margin","round","ones","rand","syn","ceil","floor","size","clear","zeros","eye","mean","std","cov","det","eig","inv","norm","rank","trace","expm","logm","sqrtm","linspace","plot","title","xlabel","ylabel","legend","text","grid","meshgrid","mesh","num2str","fft","ifft","arrayfun","cellfun","input","fliplr","flipud","ismember"]),p=r(["return","case","switch","else","elseif","end","endif","endfunction","if","otherwise","do","for","while","try","catch","classdef","properties","events","methods","global","persistent","endfor","endwhile","printf","sprintf","disp","until","continue","pkg"]);function f(e,t){return e.sol()||"'"!==e.peek()?(t.tokenize=d,d(e,t)):(e.next(),t.tokenize=d,"operator")}function h(e,t){return e.match(/^.*%}/)?t.tokenize=d:e.skipToEnd(),"comment"}function d(e,t){if(e.eatSpace())return null;if(e.match("%{"))return t.tokenize=h,e.skipToEnd(),"comment";if(e.match(/^[%#]/))return e.skipToEnd(),"comment";if(e.match(/^[0-9\.+-]/,!1)){if(e.match(/^[+-]?0x[0-9a-fA-F]+[ij]?/))return e.tokenize=d,"number";if(e.match(/^[+-]?\d*\.\d+([EeDd][+-]?\d+)?[ij]?/)||e.match(/^[+-]?\d+([EeDd][+-]?\d+)?[ij]?/))return"number"}if(e.match(r(["nan","NaN","inf","Inf"])))return"number";var n=e.match(/^"(?:[^"]|"")*("|$)/)||e.match(/^'(?:[^']|'')*('|$)/);return n?n[1]?"string":"error":e.match(p)?"keyword":e.match(u)?"builtin":e.match(m)?"variable":e.match(a)||e.match(o)?"operator":e.match(i)||e.match(c)||e.match(s)?null:e.match(l)?(t.tokenize=f,null):(e.next(),"error")}let k={name:"octave",startState:function(){return{tokenize:d}},token:function(e,t){var n=t.tokenize(e,t);return("number"===n||"variable"===n)&&(t.tokenize=f),n},languageData:{commentTokens:{line:"%"}}}}}]);
//# sourceMappingURL=173.fc1aba9536647a44.js.map?v=fc1aba9536647a44