"use strict";(self.rspackChunk_jupyterlab_application_top=self.rspackChunk_jupyterlab_application_top||[]).push([[2292],{56351(a,t,e){e.d(t,{diagram:()=>l});var r=e(78599),n=e(76693),p=e(70416),s=e(24010),i={parse:(0,p.K2)(async a=>{let t=await (0,s.qg)("info",a);p.Rm.debug(t)},"parse")},d=(0,p.K2)(()=>"11.15.0","getVersion"),l={parser:i,db:{getVersion:d},renderer:{draw:(0,p.K2)((a,t,e)=>{p.Rm.debug("rendering info diagram\n"+a);let s=(0,r.D)(t);(0,n.a$)(s,100,400,!0),s.append("g").append("text").attr("x",100).attr("y",40).attr("class","version").attr("font-size",32).style("text-anchor","middle").text(`v${e}`)},"draw")}}}}]);
//# sourceMappingURL=2292.e0d555b6fb8720dc.js.map?v=e0d555b6fb8720dc